// Copyright 1996, Marimba Inc. All Rights Reserved.
// Please feel free to use this code as an example.

import java.io.*;
import java.net.*;
import java.awt.*;

import marimba.io.*;
import marimba.gui.*;
import marimba.persist.*;
import marimba.desktop.*;

/**
 * A simple widget that scrolls down a set of headlines.
 *
 * @author	Arthur van Hoff
 */
public class HeadlineWidget extends Widget implements ClockClient {
    String headline[] = new String[50];
    String urls[] = new String[50];
    int pos[] = new int[50];
    String src = "data.txt";

    /**
     * Get the properties of this widget.
     */
    public void getProperties(PropertyList list) {
	super.getProperties(list);
	list.setURL("src", src, "data.txt");
    }

    /**
     * Set the properties of this widget.
     */
    public void setProperties(PropertyList list) {
	super.setProperties(list);
	src = list.getURL("src", "data.txt");
    }

    /**
     * Start
     */
    public synchronized void start() {
	try {
	    FastInputStream in = new FastInputStream(getPresentation().getURL(src).openStream());
	    String str = in.readLine();
	    int idx = str.indexOf("|");
	    //String url = in.readLine();
	    String url = "http://www.marimba.com";
	    if (idx != -1) {
		url = str.substring(idx+1);
	    }
	    str = str.substring(0, idx);
		

	    in.close();

	    System.arraycopy(headline, 0, headline, 1, headline.length-1);
	    System.arraycopy(urls, 0, urls, 1, urls.length-1);
	    System.arraycopy(pos, 0, pos, 1, pos.length-1);

	    FontMetrics fm = getFontMetrics(font);
	    headline[0] = str;
	    urls[0] = url;
	    pos[0] = -fm.getDescent();
	    Clock.master.add(this);
	} catch (IOException e) {
	    e.printStackTrace();
	}
    }

    /**
     * Scrolldown
     */
    public synchronized long tick(long tm, Object arg) {
	for (int i = 0 ; i < headline.length ; i++) {
	    pos[i] += 1;
	}
	repaint();
	return (pos[0] > getFontMetrics(font).getHeight()) ?
	    -1 : System.currentTimeMillis() + 100;
    }

    /**
     * Stop the animation.
     */
    public void stop() {
	Clock.master.remove(this);
    }

    /**
     * Restart
     */
    void restart() {
	stop();
	start();
	clearPopups();
    }

    /**
     * Paint
     */
    public synchronized void paint(Graphics g) {
	for (int i = 0 ; i < headline.length ; i++) {
	    if (headline[i] != null) {
		g.drawString(headline[i], 0, pos[i]);
	    }
	}
    }

    /**
     * Do something when the user clicks on a URL
     */
    public boolean handleEvent(Event evt) {
	switch (evt.id) {
	  case Event.MOUSE_DOWN:
	    FontMetrics fm = getFontMetrics(font);
	    int descent = fm.getDescent();
	    for (int i = 0 ; i < urls.length ; i++) {
		if (evt.y < pos[i] + descent) {
		    if (urls[i] != null) {
			Desktop.getDesktop().showURL(getPresentation().getURL(urls[i]));
		    }
		    return true;
		}
	    }
	    return true;
	}
	return super.handleEvent(evt);
    }

    /**
     * Get the correct tip.
     */
    public String getTip(int x, int y) {
	FontMetrics fm = getFontMetrics(font);
	int descent = fm.getDescent();
	for (int i = 0 ; i < urls.length ; i++) {
	    if (y < pos[i] + descent) {
		return "go to " + urls[i];
	    }
	}
	return null;
    }
}
