// Copyright 1996, Marimba Inc. All Rights Reserved.
// @(#)SameGamePlugin.java, 1.14, 01/19/97

package samegame;

import java.io.*;
import java.util.*;

import marimba.plugin.*;
import marimba.io.*;
import marimba.util.*;

/**
 * This class is the server plugin
 * to manage high scores for the Samegame channel. 
 *
 * @author      Carl W. Haynes III
 * @version     1.14, 01/19/97
 */

public class SameGamePlugin extends Plugin {

    Vector highScores = new Vector(200);
    static final String HIGHSCOREFILE = "highscores.txt";
    File highScoreFile = null;
    static final int NOT_IN_TOPTEN = -1;
    int lowestScore = 0;
    
    /** check some of last high score list generated */
    Checksum highScoreChecksum = null;

    /** keep the byte array around */
    byte highScoreBytes[];

    /** initialize */
    public synchronized void init() {
	highScoreFile = getDataFile(HIGHSCOREFILE);
	// initialize array
	for (int i = 0 ; i < HighScoreManager.MAXHIGHSCORES ; i++) {
	    HighScore hs = new HighScore();
	    highScores.addElement(hs);
	}
	readHighScores();

	highScoreBytes = scoresToBytes();
	highScoreChecksum = calculateChecksum(highScoreBytes);

    }

    /**
      * read the high scores
      */
    public void readHighScores() {
	if (highScoreFile.exists() == false) {
	    return;
	}
	try {
	    FastInputStream fis = new FastInputStream(highScoreFile);
	    String s;
	    while ((s = fis.readLine()) != null) {
		//DebugStr("reading file: " + s);
		add(s);
	    }
	    fis.close();
	}
	catch(IOException e) {
	    e.printStackTrace();
	    return;
	}
    }

    /** parse and add a new high score */
    public void add(String s) {
	StringTokenizer st = new StringTokenizer(s, "\t");
	if (st.hasMoreElements() == false)
	    return;
	String name = (String)st.nextElement();
	//-----
	if (st.hasMoreElements() == false)
	    return;
	String email = (String)st.nextElement();
	//-----
	if (st.hasMoreElements() == false)
	    return;
	String tmp = (String)st.nextElement();
	int score = 0;
	try {
	    score = Integer.parseInt(tmp);
	}
	catch(NumberFormatException e) {
	    e.printStackTrace();
	    return;
	}
	//-----
	if (st.hasMoreElements() == false)
	    return;
	tmp = (String)st.nextElement();
	int time = 0;
	try {
	    time = Integer.parseInt(tmp);
	}
	catch(NumberFormatException e) {
	    e.printStackTrace();
	    return;
	}
	Date date = new Date(time);

	HighScore hs = new HighScore(score, name, email, date);
	insert(hs);
    }
    
    /** finds location in top ten and inserts */
    public void insert(HighScore hs) {
	int i = place(hs.getScore());
	if (i == NOT_IN_TOPTEN)
	    return;
	//DebugStr("insert at: [" + i + "] / " + hs.toString());
//	Thread.dumpStack();
	insertAt(hs, i);
    }

    /** finds location in top ten but does not insert */
    public void insertAt(HighScore hs, int place) {
	highScores.insertElementAt(hs, place);
	while (highScores.size() > HighScoreManager.MAXHIGHSCORES)
	    highScores.removeElementAt(HighScoreManager.MAXHIGHSCORES);
	lowestScore = ((HighScore)highScores.elementAt(HighScoreManager.MAXHIGHSCORES-1)).getScore();
    }

    /** finds location in top ten but does not insert */
    public int place(int score) {
	if (score <= lowestScore)
	    return(NOT_IN_TOPTEN);
	
	int placement = HighScoreManager.MAXHIGHSCORES-1;
	Vector v = highScores;
	for (int i = HighScoreManager.MAXHIGHSCORES-1 ; i >= 0 ; i--) {
	    HighScore hs = (HighScore)v.elementAt(i);
	    if (score < hs.getScore())
		return(placement);
	    placement = i;
	}


	return(placement);
    }
    
    /**
     * Process a request.
     */
    public synchronized void processRequest(RequestContext context) throws IOException {
	
	Vector logs = getLogEntries(context);
	boolean highScoresHaveChanged = false;
	for (Enumeration e = logs.elements() ; e.hasMoreElements() ; ) {
	    LogEntry le = (LogEntry)e.nextElement();
	    String s = new String(le.data, 0);
	    //DebugStr("log entry: " + s);
	    add(s);
	    highScoresHaveChanged = true;
	}

	if (highScoresHaveChanged || highScoreChecksum == null) {
	    highScoreBytes = scoresToBytes();
	    highScoreChecksum = calculateChecksum(highScoreBytes);
	}

	context.addFile(HIGHSCOREFILE, highScoreBytes, highScoreChecksum, PERSISTENT);

	if (highScoreFile == null) {
	    //DebugStr("highScoreFile == null");
	}
	try {
	    if (highScoreFile.exists() == false) {
		File dir = new File(highScoreFile.getParent());
		if (dir.exists() == false) {
		    if (dir.mkdirs()){
		    }
		}
	    }

	    FastOutputStream fos = new FastOutputStream(highScoreFile);
	    for (Enumeration e = highScores.elements() ; e.hasMoreElements() ; ) {
		HighScore hs = (HighScore)e.nextElement();
		fos.println(hs.toString());
	    }
	    fos.close();
	}
	catch(IOException e) {
	    e.printStackTrace();
	}

	super.processRequest(context);

    }


    /**
      * returns byte array ready for passing back to channel
      */
    public byte[] scoresToBytes() {
	StringBuffer  buf = new StringBuffer();
	for (Enumeration e = highScores.elements() ; e.hasMoreElements() ; ) {
	    HighScore hs = (HighScore)e.nextElement();
	    buf.append(hs.toString());
	    if (e.hasMoreElements())
		buf.append("\n");
	}

	String s = new String(buf);
	int len = s.length();
	byte b[] = new byte[len];
	s.getBytes(0, len, b, 0);
	return(b);
    }

    /** debugging */
    public void DebugStr(String s) {
	debugPrint(s);
    }

}


