// Copyright 1996, Marimba Inc. All Rights Reserved.
// @(#)AppletViewer.java, 1.33, 01/10/97

package marimba.channel;

import java.util.*;
import java.net.*;
import java.awt.*;
import java.applet.Applet;
import java.applet.AudioClip;
import java.applet.AppletStub;

import marimba.persist.ClassLoaderThreadGroup;
import marimba.persist.URLClassLoader;

/**
 * A frame for a channel applet. This class is used internally
 * by the AppletViewer class.
 *
 * @author	Arthur van Hoff
 * @version 	1.33, 01/10/97
 */
public class AppletViewer extends ApplicationFrame implements AppletStub, AppletContext {
    Applet applet;

    /**
     * Return the applet.
     */
    public Applet getApplet() {
	return applet;
    }

    Dimension appletSize() {
	int width = 400;
	int height = 400;
	
	String str = getParameter("width");
	if (str != null) {
	    try {
		width = Integer.parseInt(str);
	    } catch (NumberFormatException e) {
	    }
	}
	str = getParameter("height");
	if (str != null) {
	    try {
		height = Integer.parseInt(str);
	    } catch (NumberFormatException e) {
	    }
	}
	return new Dimension(width, height);
    }

    public void pack() {
	addNotify();

	Dimension siz = appletSize();
	Insets in = insets();
	resize(in.left + siz.width + in.right, in.top + siz.height + in.bottom);

	Dimension d = size(), sd = getToolkit().getScreenSize();
	move((sd.width - d.width)/2, (sd.height - d.height)/2);
    }

    public void start() {
	setTitle((context != null) ? context.getChannelName() : getParameter("code"));
	pack();
	super.start();

	String code = getParameter("code");
	if (code == null) {
	    showAppletException(new Exception("code parameter missing"));
	    return;
	}
	if (code.endsWith(".class")) {
	    code = code.substring(0, code.length() - 6).replace('/', '.');
	}
	if (code.endsWith(".java")) {
	    code = code.substring(0, code.length() - 5).replace('/', '.');
	}

	try {
	    // Create the applet using the class loader
	    URLClassLoader loader = ClassLoaderThreadGroup.getCurrentURLClassLoader();
	    applet = (Applet)loader.loadClass(code).newInstance();
		
	    // Stick it in the frame
	    applet.setStub(this);
	    applet.hide();
	    add("Center", applet);
	    showAppletLog("applet loaded");
	    validate();

	    // Initialize the applet
	    applet.resize(appletSize());
	    applet.init();
	    showAppletLog("applet initialized");

	} catch (ClassCastException e) {
	    showAppletLog("load: " + code + " not an instance of java.applet.Applet");
	    showAppletException(e);
	} catch (ClassNotFoundException e) {
	    showAppletLog("load: class " + code + " not found");
	    showAppletException(e);
	} catch (InstantiationException e) {
	    showAppletLog("load: " + code + " can't be instantiated");
	    showAppletException(e);
	} catch (IllegalAccessException e) {
	    showAppletLog("load: " + code + " is not public or has no public constructor");
	    showAppletException(e);
	} catch (ThreadDeath e) {
	    throw e;
	} catch (Throwable e) {
	    showAppletException(e);
	}

	try {
	    if (applet != null) {
		applet.start();
		applet.show();
		validate();
		showAppletLog("applet started");
	    }
	} catch (ThreadDeath e) {
	    throw e;
	} catch (Throwable e) {
	    showAppletException(e);
	}
    }

    public void stop() {
	if (applet == null) {
	    return;
	}

	try {
	    applet.hide();
	    applet.stop();
	    showAppletLog("applet stopped");
	} catch (ThreadDeath e) {
	    throw e;
	} catch (Throwable e) {
	    showAppletException(e);
	}

	try {
	    applet.destroy();
	    showAppletLog("applet destroyed");

	    remove(applet);
	    showAppletLog("applet disposed");
	} catch (ThreadDeath e) {
	    throw e;
	} catch (Throwable e) {
	    showAppletException(e);
	}

	super.stop();
    }

    public boolean isActive() {
	return isVisible() && (applet != null);
    }
    
    public URL getDocumentBase() {
	return context.getBase();
    }

    public URL getCodeBase() {
	return context.getCodeBase();
    }

    public ApplicationContext getApplicationContext() {
	return context;
    }

    public String getParameter(String name) {
	return context.getParameter(name);
    }

    public java.applet.AppletContext getAppletContext() {
	return this;
    }

    public void appletResize(int width, int height) {
	// REMIND
    }

    public AudioClip getAudioClip(URL url) {
	return context.getAudioClip(url);
    }

    public Image getImage(URL url) {
	return context.getImage(url);
    }

    public Applet getApplet(String name) {
	return null;
    }

    public Enumeration getApplets() {
	return new Vector().elements();
    }

    public void showDocument(URL url) {
	context.showDocument(url);
    }

    public void showDocument(URL url, String target) {
	context.showDocument(url, target);
    }

    public void showStatus(String status) {
	context.showStatus(status);
    }

    protected void showAppletLog(String str) {
	//System.out.println("  [" + str + "]");
    }		

    protected void showAppletException(Throwable t) {
	t.printStackTrace();
    }

    public boolean handleEvent(Event evt) {
	switch (evt.id) {
	  case DATA_AVAILABLE_EVENT:
	  case DATA_INSTALLED_EVENT:
	    if ((applet != null) && (evt.target == this)) {
		evt.target = applet;
		return applet.handleEvent(evt);
	    }
	    break;
	}
	return super.handleEvent(evt);
    }
}
