// Copyright 1996, Marimba Inc. All Rights Reserved.
// @(#)Application.java, 1.8, 11/02/96

package marimba.channel;

import java.awt.*;

/**
 * This is the Application interface for the marimba Tuner. A class
 * which implements this interface can act as an application in the
 * context of the tuner.
 *
 * @author	Arthur van Hoff
 * @author	Maurice Balick
 * @version 	1.8, 11/02/96
 */
public interface Application {
    /*
     * An event with this ID is posted to the Application when 
     * the tuner is about to issue a request to update the channel.
     * This is an opportunity for the channel to log data.
     */
    int DATA_UPDATE_EVENT	= 3285287;

    /**
     * An event with this ID is posted to the Application when new
     * application data is available for installation. The application
     * can respond with a call requestInstall() on the ChannelApplicationContext
     * to install the newly arrived data.
     */
    int DATA_AVAILABLE_EVENT 	= 3285288;

    /*
     * An event with this ID is posted to the Application when 
     * application data has been installed. The application should
     * reload its data files in response.
     */
    int DATA_INSTALLED_EVENT	= 3285289;

    /*
     * An event with this ID is posted to the Application when 
     * application data has been installed, the application was
     * not running, and the application's "install.inactive" property
     * is set to "notify". The application is automatically started
     * when the event handler returns true.
     */
    int DATA_NOTIFY_EVENT	= 3285290;

    /**
     * This methods is called by the system to initialize the ChannelApplication.
     * @param context a lot of information about this channel including
     *			simple utilities to help implement this interface.
     */
    void setContext(ApplicationContext context);

    /**
     * Start the application. This method is called by the
     * tuner to notify that the application is started.
     */
    void start();

    /**
     * Stop the applicaton. This method is called by the tuner
     * to notify the application that it will be killed. To stop
     * an application call stop() in the ChannelApplicationContext.
     */
    void stop();

    /**
     * Handle an event. Events are posted to the application to
     * notify applications of changes in its status.
     */
    boolean handleEvent(Event evt);
}

