// Copyright 1996, Marimba Inc. All Rights Reserved.
// @(#)ApplicationFrame.java, 1.4, 09/28/96

package marimba.channel;

import java.net.*;
import java.awt.*;

/**
 * A helper class for creating channel applications. This
 * class creates and shows, and destroys an AWT frame as
 * the base frame of the application.
 *
 * @author	Arthur van Hoff
 * @version 	1.4, 09/28/96
 */
public class ApplicationFrame extends Frame implements Application {
    /**
     * The context of this application.
     */
    protected ApplicationContext context;
    
    /**
     * This methods is called by the system to initialize the Application.
     */
    public void setContext(ApplicationContext context) {
	this.context = context;
    }

    /**
     * Get the ApplicationContext used by this Channel.
     */
    public ApplicationContext getContext() {
	return context;
    }
    
    /**
     * Initialize the Channel, show the base frame.
     */
    public void start() {
	show();
    }

    /**
     * Destroy the Channel, hide and dispose the base frame.
     */
    public void stop() {
	hide();
	dispose();
    }

    /**
     * Updated data is available. By default, if property "update.action"
     * is "restart", then the entire channel is restarted (with the new
     * updated data).
     */
    public void notifyAvailable(String dir) {
	if ("restart".equals(context.getParameter("update.action"))) {
	    context.restart();
	} else if ("install".equals(context.getParameter("update.action"))) {
	    context.installData("");
	}
    }
    
    /**
     * Data has been installed. Called to notify the application
     * that new data has been installed. This is only called if
     * the application explicitly requests installation of new data
     * using installChannel().
     */
    public void notifyInstall(String dir) {
    }

    /**
     * Event handler for Window Management events: Asks for the
     * channel to be stopped (and soon destroyed) when the presentation
     * is closed.
     */
    public boolean handleEvent(Event evt) {
	switch (evt.id) {
	  case DATA_AVAILABLE_EVENT:
	    notifyAvailable((String)evt.arg);
	    return true;
	    
	  case DATA_INSTALLED_EVENT:
	    notifyInstall((String)evt.arg);
	    return true;

	  case Event.WINDOW_DESTROY:
	    context.stop();
	    return true;
	}
	return super.handleEvent(evt);
    }
}
