// Copyright 1996, Marimba Inc. All Rights Reserved.
// @(#)ApplicationPlayerFrame.java, 1.6, 09/28/96

package marimba.channel;

import java.net.*;
import java.awt.*;

import marimba.gui.*;

/**
 * A presentation player for channels. To view a presentation in a channel,
 * specify the channel properties as follows:
 * <pre>
 *	main=marimba.channel.ApplicationPlayerFrame
 *	presentation=foobar.gui
 * </pre>
 *
 * @author	Arthur van Hoff
 * @author	Maurice Balick 
 * @version 	1.6, 09/28/96
 */
public class ApplicationPlayerFrame extends PlayerFrame implements Application {
    /**
     * The context of this application.
     */
    protected ApplicationContext context;
    
    /**
     * Create a new ApplicationPlayerFrame.
     */
    public ApplicationPlayerFrame() {
	super(false);
    }
    
    /**
     * This methods is called by the system to initialize the Application.
     * It will try to find and load the 'presentation' specified in
     * the channel properties.
     */
    public void setContext(ApplicationContext context) {
	this.context = context;

	// Reset the context
	add("Center", player = createPlayerPanel(context));
	util = new PlayerUtil(player);

	// Load the presentation
	String presentation = context.getParameter("presentation");
	if (presentation != null) {
	    try {
		util.setPresentation(new URL(context.getBase(), presentation));
	    } catch (MalformedURLException e) {
		e.printStackTrace();
	    }
	}
    }

    /**
     * Get the ApplicationContext used by this Application.
     */
    public ApplicationContext getContext() {
	return context;
    }
    
    /**
     * Initialize the Channel.
     */
    public void start() {
	show();
    }

    /**
     * Destroy the Channel.
     */
    public void stop() {
	hide();
	dispose();
    }

    /**
     * Updated data is available. By default, if property "update.action"
     * is "restart", then the entire channel is restarted (with the new
     * updated data).
     */
    public void notifyAvailable(String dir) {
	if ("restart".equals(context.getParameter("update.action"))) {
	    context.restart();
	} else if ("install".equals(context.getParameter("update.action"))) {
	    context.installData("");
	}
    }
    
    /**
     * Data has been installed. Called to notify the application
     * that new data has been installed. This is only called if
     * the application explicitly requests installation of new data
     * using install().
     */
    public void notifyInstall(String dir) {
    }

    /**
     * Event handler for Window Management events: Asks for the
     * channel to be stopped (and soon destroyed) when the presentation
     * is closed.
     */
    public boolean handleEvent(Event evt) {
	switch (evt.id) {
	  case DATA_AVAILABLE_EVENT:
	    notifyAvailable((String)evt.arg);
	    return true;
	    
	  case DATA_INSTALLED_EVENT:
	    notifyInstall((String)evt.arg);
	    return true;

	  case Event.WINDOW_DESTROY:
	    context.stop();
	    return true;
	}
	return super.handleEvent(evt);
    }
}
